@@ -3,9 +3,12 @@ import concurrent.futures
 import json
 import logging
 import ssl
+import time
 import traceback
 from functools import partial
+from uuid import uuid4
 
+import aioredis
 import boto3
 from aiokafka import AIOKafkaConsumer, AIOKafkaProducer
 
@@ -29,8 +32,10 @@ class Worker:
         sasl_opts={},
         consumer_opts={},
         producer_opts={},
+        redis_opts={},
         sqs_opts={},
     ):
+        self._uuid = str(uuid4())
         try:
             # only called in async func when the worker is created
             loop = asyncio.get_running_loop()
@@ -54,11 +59,11 @@ class Worker:
         #       AWS_DEFAULT_REGION
         # https://boto3.amazonaws.com/v1/documentation/api/latest/guide/configuration.html#environment-variables
         try:
+            self._sqs_opts = sqs_opts
             self._sf_client = boto3.client("stepfunctions")
 
             # only initiate here (low footprint), whether to use is implemented in on_run by subclass
             self._sqs_client = boto3.client("sqs")
-            self._sqs_opts = sqs_opts
         except Exception as e:
             logger.warning(f"Unable to connect to boto3 client: {e}")
 
@@ -71,6 +76,12 @@ class Worker:
                 loop, kafka_broker_addr, sasl_opts, producer_opts
             )
 
+        # optional redis
+        self._redis_opts = redis_opts
+        self._redis = None
+        self._redis_retry = 0
+        self._redis_reconnecting = False
+
         # legacy code, to be removed in future releases
         if on_run:
             logger.warning(
@@ -143,6 +154,47 @@ class Worker:
         logger.info("Producer connected to kafka on {}".format(broker_addr))
         return producer
 
+    async def create_redis(self):
+        """
+        Keep retrying until succeed every 5 * retry seconds, max 1 minute interval
+        """
+        self._redis_reconnecting = True
+        while True:
+            try:
+                if self._redis_opts:
+                    self._redis = await aioredis.create_redis_pool(
+                        self._redis_opts["url"], timeout=5
+                    )
+
+                    logger.warning("Redis connected")
+                else:
+                    logger.warning("Redis opts not supplied")
+                break
+            except Exception as e:
+                # failed to create, just retry
+                logger.error(e)
+            self._redis_retry += 1
+            await asyncio.sleep(min(60, 5 * self._redis_retry))
+            logger.warning(f"Retry connecting redis {self._redis_retry}")
+
+        # reset counters and flags after connection
+        self._redis_retry = 0
+        self._redis_reconnecting = False
+
+    async def destroy_redis(self):
+        """
+        Don't care if the close fails or timeout
+        """
+        if self._redis is not None and not self._redis_reconnecting:
+            try:
+                self._redis.close()
+                await asyncio.wait_for(self._redis.wait_closed(), timeout=3)
+            except Exception as e:
+                logger.error(e)
+            finally:
+                self._redis = None
+                logger.warning("Redis destoryed")
+
     async def __aenter__(self):
         """
         Async iterator-enter coroutine
@@ -161,6 +213,7 @@ class Worker:
         return self._exit
 
     async def start(self):
+        await self.create_redis()
 
         if self._consumer:
             if self._consumer_topic:
@@ -190,6 +243,7 @@ class Worker:
         logger.warning("clean up after run")
         self._exit = True
         await self.stop_kafka(True)
+        await self.destroy_redis()
 
     async def run(self):
         """
@@ -283,35 +337,69 @@ class Worker:
             logger.error(f"Unable to send step functions response: {e}")
             return
 
-    async def send_result(self, result, context=None, kafka_result_topic=None):
+    async def send_result(self, result, context=None, result_channel_or_topic=None):
         context_topic = None
         result_data = json.dumps(result).encode("utf-8")
 
-        if context is not None and context.get("step_functions_task_id") is not None:
-            await self.send_step_functions_result(
-                context.get("step_functions_task_id"), result, result_data
-            )
+        if context:
+            if context.get("step_functions_task_id") is not None:
+                await self.send_step_functions_result(
+                    context.get("step_functions_task_id"), result, result_data
+                )
 
-        # step function and kafka callback topic don't need to be mutually exclusive
-        if context and context.get("topic"):
-            context_topic = context.get("topic")
-            partition = context.get("partition")
-            headers = context.get("headers")
-            if headers:
-                headers = [(k, v.encode("utf-8")) for k, v in headers.items()]
+            # step function and kafka callback topic don't need to be mutually exclusive
+            if context.get("topic") and self._producer:
+                context_topic = context.get("topic")
+                partition = context.get("partition")
+                headers = context.get("headers")
+                if headers:
+                    headers = [(k, v.encode("utf-8")) for k, v in headers.items()]
 
-            await self._producer.send(
-                context_topic, result_data, partition=partition, headers=headers
-            )
+                await self._producer.send(
+                    context_topic, result_data, partition=partition, headers=headers
+                )
 
-            logger.info(
-                "pushing result with code {} "
-                'to topic "{}"'.format(result.get("code", None), kafka_result_topic)
-            )
+                logger.info(
+                    "pushing result with code {} "
+                    'to topic "{}"'.format(result.get("code", None), context_topic)
+                )
 
-        # always check the kafka_result_topic as it is usually used for data persistence
-        if kafka_result_topic and context_topic != kafka_result_topic:
-            await self._producer.send(kafka_result_topic, result_data)
+            elif self._redis and context.get("redis_channel"):
+                redis_channel = context.get("redis_channel")
+
+                await self._redis.publish(redis_channel, result_data)
+
+                logger.info(
+                    "pushing result with code {} "
+                    'to redis "{}"'.format(result.get("code", None), redis_channel)
+                )
+
+        # always check the result_channel_or_topic as it is usually used for data persistence
+        if result_channel_or_topic:
+            if result_channel_or_topic.startswith("https://sqs"):
+                if result_channel_or_topic.endswith(".fifo"):
+                    nowait_delete = asyncio.get_running_loop().run_in_executor(
+                        None,
+                        partial(
+                            self._sqs_client.delete_message,
+                            QueueUrl=result_channel_or_topic,
+                            MessageBody=result_data,
+                            MessageDeduplicationId=f"{time.time()}",
+                            MessageGroupId=self._uuid,
+                        ),
+                    )
+                else:
+                    nowait_delete = asyncio.get_running_loop().run_in_executor(
+                        None,
+                        partial(
+                            self._sqs_client.delete_message,
+                            QueueUrl=result_channel_or_topic,
+                            MessageBody=result_data,
+                        ),
+                    )
+                asyncio.ensure_future(nowait_delete)
+            elif self._producer and context_topic != result_channel_or_topic:
+                await self._producer.send(result_channel_or_topic, result_data)
 
         # unify error logging
         self.capture_error(result=result)
